open Instr

let stdin = Data.alloc ()
let stdout = Data.alloc ()
let stderr = Data.alloc ()
let int_format = Data.create "%d"
let float_format = Data.create "%.12g"
let true_string = Data.create "true"
let false_string = Data.create "false"

let gen_string_of_int () =
  Code.add_list [
    Push;
    Getglobal int_format;
    Ccall (2, Prim.add "caml_format_int");
  ]
;;

let gen_string_of_float () =
  Code.add_list [
    Push;
    Getglobal float_format;
    Ccall (2, Prim.add "caml_format_float");
  ]
;;

let gen_string_of_bool () =
  let ptr0 = Code.create_addr () in
  let ptr1 = Code.create_addr () in
    Code.add (Branchif ptr0);
    Code.add (Getglobal false_string);
    Code.add (Branch ptr1);
    Code.register_addr ptr0;
    Code.add (Getglobal true_string);
    Code.register_addr ptr1;
;;

let gen_output_string () =
  Code.add_list [
    Push; Acc 1;
    Ccall (1, Prim.add "caml_ml_string_length");
    Push; Const 0;
    Push; Acc 3;
    Push; Acc 3;
    Ccall (4, Prim.add "caml_ml_output");
    Pop 2;
  ]
;;

let gen_print_string () =
  Code.add_list [
    Push;
    Getglobal stdout;
  ];
  gen_output_string ();
;;

let gen_println () =
  gen_print_string ();
  Prim.output_char stdout '\n';
;;

let gen_concat_string () =
  Code.add_list [
    Push;
    Ccall (1, Prim.add "caml_ml_string_length");
    Push; Acc 2;
    Ccall (1, Prim.add "caml_ml_string_length");
    Push; Acc 0;
    Push; Acc 2;
    Addint;
    Ccall (1, Prim.add "caml_create_string");
    Push; Acc 2;
    Push; Const 0;
    Push; Acc 2;
    Push; Const 0;
    Push; Acc 7;
    Ccall (5, Prim.add "caml_blit_string");
    Acc 1;
    Push; Acc 3;
    Push; Acc 2;
    Push; Const 0;
    Push; Acc 8;
    Ccall (5, Prim.add "caml_blit_string");
    Acc 0;
    Pop 5;
  ]
;;

let gen_cast cast =
  match cast with
    | Ast_typer.Float_of_int ->
	Code.add (Ccall (1, Prim.add "caml_float_of_int"))
    | Ast_typer.Int_of_float ->
	Code.add (Ccall (1, Prim.add "caml_int_of_float"))
    | Ast_typer.Int_of_int -> ()
    | Ast_typer.String_of_int ->
	Code.add_list [
	  Push;
	  Getglobal int_format;
	  Ccall (2, Prim.add "caml_format_int");
	]
    | Ast_typer.String_of_float ->
	Code.add_list [
	  Push;
	  Getglobal float_format;
	  Ccall (2, Prim.add "caml_format_float");
	]
    | Ast_typer.String_of_bool ->
	let ptr0 = Code.create_addr () in
	let ptr1 = Code.create_addr () in
	  Code.add (Branchif ptr0);
	  Code.add (Getglobal false_string);
	  Code.add (Branch ptr1);
	  Code.register_addr ptr0;
	  Code.add (Getglobal true_string);
	  Code.register_addr ptr1;
    | Ast_typer.String_of_object ->
	assert false
;;
